@@ -3,6 +3,7 @@ from .chart_tool import get_pie, get_bar, get_line, get_scatter
 from .chart_tool import get_bar3d, get_line3d, get_scatter3d
 from .chart_tool import get_boxplot, get_funnel, get_geo, get_map
 from .chart_tool import timeline_decorator, by_decorator
+from .data_tool import infer_dtype, _categorize_array
 
 
 @pd.api.extensions.register_dataframe_accessor("echart")
@@ -11,12 +12,13 @@ class DataFrameEcharts:
         self._obj = pandas_obj
 
     def pie(self,
-            x="",
-            y="",
+            x,
+            y,
             title="",
             subtitle="",
             agg_func=None,
             label_show=False,
+            label_opts={},
             legend_opts={},
             theme=None,
             by=None,
@@ -34,14 +36,15 @@ class DataFrameEcharts:
             title=title,
             subtitle=subtitle,
             label_show=label_show,
+            label_opts=label_opts,
             agg_func=agg_func,
             legend_opts=legend_opts,
             theme=theme,
         )
 
     def bar(self,
-            x="",
-            ys="",
+            x,
+            ys,
             xaxis_name=None,
             yaxis_name="",
             title="",
@@ -49,6 +52,7 @@ class DataFrameEcharts:
             agg_func=None,
             stack_view=False,
             label_show=False,
+            label_opts={},
             reverse_axis=False,
             legend_opts={},
             theme=None,
@@ -56,6 +60,10 @@ class DataFrameEcharts:
             timeline=None,
             timeline_opts={}):
         df = self._obj.copy()
+        # 由于dataframe的bar的x轴可以只考虑离散值，所以先按照
+        # x排序，然后将x转为字符串类型，注意要在转str前排序，要不然
+        # 会按照字典排序，从而造成数字排序很奇怪
+        df = df.sort_values(by=x)
         df[x] = df[x].astype(str)
 
         if xaxis_name is None:
@@ -78,14 +86,15 @@ class DataFrameEcharts:
             stack_view=stack_view,
             reverse_axis=reverse_axis,
             label_show=label_show,
+            label_opts=label_opts,
             legend_opts=legend_opts,
             theme=theme
         )
 
     def bar3d(self,
-              x="",
-              y="",
-              z="",
+              x,
+              y,
+              z,
               xaxis_name=None,
               yaxis_name=None,
               zaxis_name=None,
@@ -121,8 +130,8 @@ class DataFrameEcharts:
         )
 
     def line(self,
-             x="",
-             ys="",
+             x,
+             ys,
              xtype=None,
              xaxis_name=None,
              yaxis_name="",
@@ -131,14 +140,13 @@ class DataFrameEcharts:
              agg_func=None,
              smooth=False,
              label_show=False,
+             label_opts={},
              legend_opts={},
              theme=None,
              by=None,
              timeline=None,
              timeline_opts={}):
         df = self._obj.copy()
-        df[x] = df[x].astype(str)
-
         if xaxis_name is None:
             xaxis_name = x
 
@@ -159,14 +167,15 @@ class DataFrameEcharts:
             agg_func=agg_func,
             smooth=smooth,
             label_show=label_show,
+            label_opts=label_opts,
             legend_opts=legend_opts,
             theme=theme
         )
 
     def line3d(self,
-               x="",
-               y="",
-               z="",
+               x,
+               y,
+               z,
                xtype=None,
                ytype=None,
                ztype=None,
@@ -207,8 +216,8 @@ class DataFrameEcharts:
         )
 
     def scatter(self,
-                x="",
-                ys="",
+                x,
+                ys,
                 xtype=None,
                 xaxis_name=None,
                 yaxis_name="",
@@ -216,6 +225,7 @@ class DataFrameEcharts:
                 subtitle="",
                 agg_func=None,
                 label_show=False,
+                label_opts={},
                 legend_opts={},
                 visualmap=False,
                 visualmap_opts={},
@@ -245,6 +255,7 @@ class DataFrameEcharts:
             subtitle=subtitle,
             agg_func=agg_func,
             label_show=label_show,
+            label_opts=label_opts,
             legend_opts=legend_opts,
             visualmap=visualmap,
             visualmap_opts=visualmap_opts,
@@ -252,9 +263,9 @@ class DataFrameEcharts:
         )
 
     def scatter3d(self,
-                  x="",
-                  y="",
-                  z="",
+                  x,
+                  y,
+                  z,
                   xtype=None,
                   ytype=None,
                   ztype=None,
@@ -295,7 +306,7 @@ class DataFrameEcharts:
         )
 
     def boxplot(self,
-                ys="",
+                ys,
                 xaxis_name="",
                 yaxis_name="",
                 title="",
@@ -324,8 +335,8 @@ class DataFrameEcharts:
         )
 
     def funnel(self,
-               x="",
-               y="",
+               x,
+               y,
                title="",
                subtitle="",
                ascending=False,
@@ -351,8 +362,8 @@ class DataFrameEcharts:
         )
 
     def geo(self,
-            x="",
-            ys="",
+            x,
+            ys,
             maptype="",
             title="",
             subtitle="",
@@ -385,8 +396,8 @@ class DataFrameEcharts:
         )
 
     def map(self,
-            x="",
-            y="",
+            x,
+            y,
             maptype="",
             title="",
             subtitle="",
@@ -421,10 +432,173 @@ class DataFrameEcharts:
 class SeriesEcharts:
     def __init__(self, series_obj):
         self._obj = series_obj
-        # TODO: series 需要区分数据是离散类型还是连续类型
-        # TODO: series 只支持画单个变量的分布图，
-        # 对于连续变量可以使用freedman_diaconis规则获取bins个数，参考自seaborn里的distplot
-        # TODO: bar和line可以支持显示density还是count，类似numpy.histogram
 
-    def pie():
-        pass
+    def _get_dist(self, dtype, bins):
+        df = self._obj.copy().to_frame()
+        xcol = df.columns[0]
+
+        if dtype is None:
+            dtype = infer_dtype(df[xcol])
+        if dtype == "value":
+            # TODO: 当数据本身不超过max_bins的大小还需要调用这个函数
+            df[xcol] = _categorize_array(df[xcol].values.tolist(), bins=bins)
+            df = df.sort_values(by=xcol)
+
+        ycol = "count_" if xcol == "count" else "count"
+        df[ycol] = 1
+        return df, xcol, ycol
+
+    def pie(self,
+            xtype=None,
+            bins=None,
+            title="",
+            subtitle="",
+            label_show=False,
+            label_opts={},
+            legend_opts={},
+            theme=None):
+        df, xcol, ycol = self._get_dist(xtype, bins)
+        return get_pie(
+            df,
+            xcol,
+            ycol,
+            title=title,
+            subtitle=subtitle,
+            label_show=label_show,
+            label_opts=label_opts,
+            agg_func='sum',
+            legend_opts=legend_opts,
+            theme=theme,
+        )
+
+    def bar(self,
+            xtype=None,
+            bins=None,
+            xaxis_name=None,
+            yaxis_name="count",
+            title="",
+            subtitle="",
+            reverse_axis=False,
+            label_show=False,
+            label_opts={},
+            legend_opts={},
+            theme=None):
+        df, xcol, ycol = self._get_dist(xtype, bins)
+        if xaxis_name is None:
+            xaxis_name = str(xcol)
+        return get_bar(
+            df,
+            xcol,
+            [ycol],
+            xaxis_name=xaxis_name,
+            yaxis_name=yaxis_name,
+            title=title,
+            subtitle=subtitle,
+            label_show=label_show,
+            label_opts=label_opts,
+            agg_func='sum',
+            stack_view=False,
+            reverse_axis=reverse_axis,
+            legend_opts=legend_opts,
+            theme=theme,
+        )
+
+    def line(self,
+             xtype=None,
+             bins=None,
+             xaxis_name=None,
+             yaxis_name="count",
+             title=None,
+             subtitle=None,
+             smooth=False,
+             label_show=False,
+             label_opts={},
+             legend_opts={},
+             theme=None):
+        df, xcol, ycol = self._get_dist(xtype, bins)
+        if xaxis_name is None:
+            xaxis_name = str(xcol)
+        return get_line(
+            df,
+            xcol,
+            [ycol],
+            xtype=xtype,
+            xaxis_name=xaxis_name,
+            yaxis_name=yaxis_name,
+            title=title,
+            subtitle=subtitle,
+            agg_func="sum",
+            smooth=smooth,
+            label_show=label_show,
+            label_opts=label_opts,
+            legend_opts=legend_opts,
+            theme=theme
+        )
+
+    def boxplot(self,
+                xaxis_name=None,
+                yaxis_name="",
+                title="",
+                subtitle="",
+                legend_opts={},
+                theme=None):
+        df = self._obj.to_frame()
+        xcol = df.columns[0]
+        if xaxis_name is None:
+            xaxis_name = str(xcol)
+        return get_boxplot(
+                df,
+                [xcol],
+                xaxis_name=xaxis_name,
+                yaxis_name=yaxis_name,
+                title=title,
+                subtitle=subtitle,
+                legend_opts=legend_opts,
+                theme=theme
+        )
+
+    def geo(self,
+            maptype,
+            title="",
+            subtitle="",
+            label_show=False,
+            visualmap=False,
+            visualmap_opts={},
+            theme=None):
+        df, xcol, ycol = self._get_dist('category', None)
+        return get_geo(
+            df,
+            xcol,
+            [ycol],
+            maptype=maptype,
+            title=title,
+            subtitle=subtitle,
+            agg_func='sum',
+            label_show=label_show,
+            visualmap=visualmap,
+            visualmap_opts=visualmap_opts,
+            theme=theme
+        )
+
+    def map(self,
+            maptype,
+            title="",
+            subtitle="",
+            label_show=False,
+            visualmap=False,
+            visualmap_opts={},
+            theme=None):
+        df, xcol, ycol = self._get_dist('category', None)
+        return get_map(
+            df,
+            xcol,
+            ycol,
+            maptype=maptype,
+            title=title,
+            subtitle=subtitle,
+            agg_func='sum',
+            label_show=label_show,
+            visualmap=visualmap,
+            visualmap_opts=visualmap_opts,
+            theme=theme
+        )
